import invariant from 'invariant';
import { Store } from '../Store';
import toMap from './toMap';

import { Iterable, OrderedMap, Map } from 'immutable';

import { STORE_INSERT, STORE_REMOVE, STORE_FILTER, STORE_UPDATE } from '../constants/StoreConstants';

export default function createReducerForStore(store) {
  invariant(store && Store.isStore(store), 'Reducer: `store` is expected to be an Orchestra.Store.');

  var identifier = store.identifier;

  var pre = store.getPre();

  return function storeReducer() {
    var state = arguments.length <= 0 || arguments[0] === undefined ? new OrderedMap() : arguments[0];
    var action = arguments[1];

    if (identifier !== action.identifier) {
      return state;
    }

    var type = action.type;
    var payload = action.payload;
    var selector = action.selector;

    switch (type) {
      case STORE_INSERT:
        {
          if (Iterable.isKeyed(payload)) {
            var item = toMap(pre(payload));
            var id = item.get('id');
            return state.set(id, item);
          }

          return state.withMutations(function (map) {
            payload.forEach(function (value) {
              var item = toMap(pre(value));
              var id = item.get('id');
              map.set(id, item);
            });
          });
        }

      case STORE_REMOVE:
        {
          if (typeof payload === 'string') {
            return state.delete(payload);
          }

          var _id = payload.get('id');
          return state.delete(_id);
        }

      case STORE_FILTER:
        {
          return state.filter(selector);
        }

      case STORE_UPDATE:
        {
          return state.map(function (x) {
            return pre(selector(x));
          });
        }

      default:
        return state;
    }
  };
}